#include "Gesture.h"



Gesture::Gesture(void)
{
	Animation = 0;
	SideOfBody = UNDEFINED;
}

Gesture::Gesture(Gesture& gesture)
{
	PoseVector = gesture.PoseVector;
	SkeletonAnimation* animation = new SkeletonAnimation();

	// we need to copy the animation by hand
	for(unsigned int i = 0; i < gesture.getSkeletonAnimation()->getSize(); i++)
	{
		animation->addFrame(gesture.getSkeletonAnimation()->getFrame(i));
	}
	Animation = animation;
	SideOfBody = gesture.getGestureType();
}


Gesture::~Gesture(void)
{
	delete Animation;
	PoseVector.clear();
}


void Gesture::clear()
{
	PoseVector.clear();
}

void Gesture::addPose(Pose& pose)
{
	PoseVector.push_back(pose);
}

Pose& Gesture::getPose(int index)
{
	return PoseVector[index];
}

std::vector<Pose>& Gesture::getPoses()
{
	return PoseVector;
}

Pose& Gesture::getFirstPose()
{
	return PoseVector[0];
}

Pose& Gesture::getLastPose()
{
	return PoseVector[PoseVector.size()-1];
}

unsigned int Gesture::getSize()
{
	return PoseVector.size();
}

void Gesture::setAnimation(SkeletonAnimation* ani)
{
	Animation = ani;
}

SkeletonAnimation* Gesture::getSkeletonAnimation()
{
	return Animation;
}

void Gesture::saveGesture(std::ofstream &open_file)
{
	int nr_of_poses = PoseVector.size();
	
	// at first we save the poses count
	open_file.write( (char*) &nr_of_poses, sizeof(int));

	for (int i = 0; i < nr_of_poses; i++)
	{
		// save all the poses
		PoseVector[i].savePose(open_file);
	}

	// save the skeleton animation (with its own routine)
	Animation->saveAnimation(open_file);

	// save the gesture type
	open_file.write( (char*) &SideOfBody, sizeof(GestureType));
}

void  Gesture::loadGesture(std::ifstream &open_file)
{
	// clear pose vector
	PoseVector.clear();

	// clear animation
	delete Animation;

	int nr_of_poses;		
	// first we read the pose count
	open_file.read((char *) &nr_of_poses, sizeof(int));

	//then we iterate over all stored data
	for (int i = 0; i < nr_of_poses; i++)
	{
		//create new pose and push it in the list
		Pose temp;
		temp.loadPose(open_file);
		PoseVector.push_back(temp);			
	}

	//create a new animation
	SkeletonAnimation* new_animation = new SkeletonAnimation();

	//load animation data
	new_animation->loadAnimation(open_file);
	
	// assign the animation
	Animation = new_animation;

	// read the gesture type
	open_file.read( (char*) &SideOfBody, sizeof(GestureType));
}