/* 
Class providing gesture structures as series of poses.
Every gesture has it's own skeleton animation.

*/
#pragma once

#include <vector>
#include "Pose.h"
#include "SkeletonAnimation.h"

// we distinguish between gestures for left, right and both sides of the body
enum GestureType{LEFT, RIGHT, BOTH, UNDEFINED};


class Gesture
{
public:

	Gesture(void);
	Gesture(Gesture& gesture);
	~Gesture(void);

	// clears all the poses of the gesture
	void  clear();
	// adds a pose to the end of the poses list
	void  addPose(Pose& pose);
	// returns a particular pose for given index
	Pose& getPose(int index);
	// returns reference to all the poses
	std::vector<Pose>& getPoses();
	Pose& getFirstPose();
	Pose& getLastPose();
	// returns the size of the poses list (not necessarily equal to  the number of skeletal animation frames)
	unsigned int getSize();
	
	// set and get body side
	void setGestureType(GestureType side_of_body){SideOfBody = side_of_body;};
	GestureType getGestureType(){return SideOfBody;};

	// set/get skeletal animation
	void  setAnimation(SkeletonAnimation* ani);
	SkeletonAnimation* getSkeletonAnimation();

	// save/load gesture to disk
	void  saveGesture(std::ofstream &open_file);
	void  loadGesture(std::ifstream &open_file);

private:
	// list containing all the poses in a gesture
	std::vector<Pose> PoseVector;
	// animation of corresponding skeletal user representation
	SkeletonAnimation* Animation;
	// which side of body
	GestureType SideOfBody;
};

