/*
Simple mesh class providing a rectangular grid with constant cell size, including rendering and calculation of local coordinates from screen coordinates.
*/

#pragma once


#include <vector>
#include "math\Math.h"

struct Face
{
	long ID;
	// indices of adjacent points
	long p0, p1, p2;
	// face normal
	math::Vec3f normal;
};

struct GridPoint
{
	long index;
	math::Vec3f coords;
	// indices of adjacent faces
	std::vector<int> faces;
	// adjacent grid points
	std::vector<int> neighbours;
	math::Vec3f normal;
	math::Vec2f tex_coords;
};


class GridMesh
{
public:

	GridMesh(int width=256, int length=256);
	~GridMesh(void);

	void render();
	// calculate local coordinates of given screen position using colorpicking
	bool ScreenToLocalCoords(math::Vec2f screen_coords, math::Vec3f& local_coords, bool snap_to_grid = true);

	GridPoint* getGridPoint(int x, int z);
	GridPoint* getGridPoint(int index){return &(PointList[index]);};
	int getWidth(){return Width;};
	int getLength(){return Length;};

	// collects grid points for normal recalculation, actual calculation takes place later in executeNormalRecalculation()
	void recalculateGridPointNormal(GridPoint* p);
	// normal recalculation
	void executeNormalsRecalculation();
	// show surface normals
	void toggleDisplaySurfaceNormals(){ShowNormals = !ShowNormals;};

	void drawColorMap();
	int getTextureID(){return TextureID;};
	void saveColorAndHeightMap();


private:

	// calculate normals of all triangle faces and grid points
	void calculateNormals();
	// calculate vertex normal for given point index
	void setVertexNormal(int point_index);
	// calculate face normal for given face index
	void setFaceNormal(int face_index);
	// set adjacent neighbour face as well as corresponding grid points
	void setNeighbours(int point_index, int face_index);
	// set a face into the face index list of a grid point
	bool setAdjacentFace(int point_index, int face_index);
	// set a grid point (referrenced by neighbour_index) into the neighbour index list of a grid point (referrenced by point_index)
	void setAdjacentGridPoint(int point_index, int neighbour_index);
	// get the index of a grid point
	long getGridPointIndex(int x, int z);

	// length of grid
	int Length;
	// width of grid
	int Width;
	// list of grid points
	std::vector<GridPoint> PointList;
	// the normals of the grid points
	std::vector<float> NormalMap;
	// list of faces
	std::vector<Face> FaceList;
	// last selected face index
	long LastSelectedFaceIndex;
	// last selected vertex index
	long LastSelectedVertexIndex;
	// texture id
	int TextureID;
	// texture data

	// normals display
	bool ShowNormals;
	// list of grid points for normal recalculation
	std::vector<GridPoint*> PointsForNormalRecalculation;
};