/*

- Demo Application -

We can use different tools to model virtual landscapes basd on a rectangular grid.
These tools are chosen via body gesture events and executed via hand gestures.

*/
#pragma once


#include "KinectSensor.h"
#include "common\glui.h"
#include <GL\freeglut.h>
#include <vector>
#include <string>

#include "WindowProperties.h"
#include "GestureRecognizer.h"
#include "HandRecognizer.h"
#include "KinectPointer.h"
#include "TextureTextRenderer.h"
#include "Tools\Colorpicker.h"
#include "Tools\NullTool.h"
#include "Tools\OrbitManipulator.h"
#include "Tools\Camera.h"
#include "Tools\TerraBrush.h"
#include "KinectTerrain\GridMesh.h"
#include "Tools\SmoothBrush.h"
#include "Tools\RandomizeBrush.h"
#include "Tools\BulldozeBrush.h"
#include "Tools\DumpBrush.h"
#include "Tools\TerraColorizer.h"
#include "CircleButton.h"
#include "FileHandling.h"
#include "SOIL.h"


class KinectTerrain
{
public:
	KinectTerrain(void);
	~KinectTerrain(void);

	// sets the static instance 
	void setInstance(KinectTerrain* app){instance = app;};
	// inits program parameters and glut entities
	void init(int argc, char** argv);
	// invokes the glut main loop
	void run();


private:

	// we have to provide a static instance of the class in order to use the static glut callback structure
	static KinectTerrain* instance;

	// GLUT CALLBACK WRAPPERS
	static void idleWrapper();
	static void updateWrapper();
	static void reshapeWindowWrapper(int width, int height);
	static void handleKeyEventWrapper(unsigned char key, int x, int y);
	static void handleSpecialKeyEventWrapper(int key, int x, int y);
	
	// CALLBACKS
	void idle();
	void update();
	void reshapeWindow(int width, int height);
	void handleKeyEvent(unsigned char key, int x, int y);
	void handleSpecialKeyEvent(int key, int x, int y);
	void handleHandEvent(std::string ev);
	void handleGestureEvent(std::string ev);
	void handleButtonEvent(std::string ev);

	// render methods
	void render();
	void renderMesh();
	void renderButtonMenu();

	// tool update 
	void updateActiveTool(Pose& pose);
	// state update of buttons in a list with given pointer coordinates
	void KinectTerrain::updateButtons(std::vector<CircleButton*>& list, math::Vec2f& coords);
	// sets the positions of the buttons in the menu
	void setButtonPositions();
	// renders a textured quad to visualize the kinect depth data (marked with user skeleton positions)
	void showDepthFrame();
	// set the background color
	void KinectTerrain::setStandardClearColor();

	// window infos
	WindowProperties WindowProps;
	int MainWindowID;

	// Kinect sensor
	KinectSensor* NuiSensor;

	// gesture recognizer
	GestureRecognizer* GRecognizer;

	// hand recognizers
	HandRecognizer* LeftHandRecognizer;
	HandRecognizer* RightHandRecognizer;

	// Pointers
	KinectPointer* LeftHandPointer;
	KinectPointer* RightHandPointer;

	// Text module to inform user e.g. about recognition
	TextureTextRenderer* InfoText;

	// List of tools
	std::vector<Tool*> ToolList;
	Tool* ActiveTool;

	// 3d camera
	Camera* camera;

	// current pose and skeleton data
	Pose CurrentPose;
	NUI_SKELETON_DATA CurrentSkeleton;

	// grid mesh
	GridMesh* Mesh;

	// shoulde we recognize gestures
	bool HandleGestures;

	// texture id for current color frame
	int ColorFrameTextureID;

	// provide animated button menu
	std::vector<CircleButton*> ButtonList;
	int AnimationCounter;
	enum AnimationState{M_INACTIVE, M_FADE_IN, M_ACTIVE, M_FADE_OUT} ButtonMenuState;

	// texture id for camera frame
	GLuint DepthTextureID;
	// flag to decide wether depth data shall be rendered
	bool RenderDepthFrame;
	
	


	//FOR DEBUGGING PURPOSES
#ifdef MOUSEDEBUG
	static void PassiveMotion(int x, int y);
	static void MouseFunc(int button, int state, int x, int y);
#endif
	
};

