#include "Brush2D.h"
#include "SOIL.h"

int Brush2D::CurrentStrokeID = 0;

Brush2D::Brush2D(KinectPointer* right_hand, std::vector<Stroke>* strokes, Manipulator2D* manipulator)
{
	RightHandPointer = right_hand;
	Strokes = strokes;
	Manipulator = manipulator;
	LineWidth = 10.0f;
	LastPoint = math::Vec2f(0.0f, 0.0f);
	Drawed = false;
	TextureID = SOIL_load_OGL_texture("../images/brushtool.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
}

Brush2D::~Brush2D()
{
}

void Brush2D::updateLeftHandInput(Pose &pose)
{
	setLineWidth(pose);
	checkStroke();
	// update pose
	LastPose = Pose(pose);
}


void Brush2D::updateRightHandInput(Pose &pose)
{
	setStrokePoint();
	// update pose
	LastPose = Pose(pose);
}	

void Brush2D::updateBothHandInput(Pose &pose)
{
	setLineWidth(pose);
	setStrokePoint();
	// update pose
	LastPose = Pose(pose);
}
	

void Brush2D::update(Pose &pose)
{
	checkStroke();
	// update pose
	LastPose = Pose(pose);
}	

void Brush2D::render()
{
	if(RightHandPointer->isActive())
	{
		math::Vec2f coords =  RightHandPointer->getWindowCoordinates();

		glPushMatrix();
		glTranslatef(coords.x, coords.y, 0.0f);
		GLUquadric *quadric = gluNewQuadric();
		glColor3fv(Colorpicker::CurrentColor.v);
		gluDisk(quadric, 0.0, LineWidth/2.0f, 16, 1);			
		glPopMatrix();

		// free memory
		gluDeleteQuadric(quadric);
	}

}

void Brush2D::checkStroke()
{
	if(Drawed)
	{
		//if we currently drawed we set an undo info
		UndoList.push_back(Strokes->back());
		// update the undo/redo manager
		UndoRedoManager::checkIn(this);
		// clear the redo list
		RedoList.clear();

		Drawed = false;
	}
	
	//prepare a new stroke
	if( (Strokes->size() == 0) || (Strokes->back().Points.size() != 0) )
	{
		Stroke stroke;
		stroke.ID = getStrokeID();
		Strokes->push_back(stroke);
	}
	
}

void Brush2D::setStrokePoint()
{
	math::Vec2f world_coords =  RightHandPointer->getWindowCoordinates();
	math::Vec2f local_coords = Manipulator->WorldToLocalCoords(world_coords);
	Strokes->back().Color = Colorpicker::CurrentColor;
	Strokes->back().LineWidth = LineWidth;
	
	// we only set a new point if it has a specific minimum distance to the last point regarding the line width
	if( (world_coords - LastPoint).getLength() > LineWidth )
	{
		Strokes->back().Points.push_back(local_coords);
		LastPoint = world_coords;
		Drawed = true;
	}
}

void Brush2D::setLineWidth(Pose& pose)
{
	LineWidth += 50*(pose.LeftHandPosition.y - LastPose.LeftHandPosition.y);
	LineWidth = max(LineWidth, 1.0f);
}

void Brush2D::undo()
{ 
	if(UndoList.size() > 0)
	{
		//clear stroke with same id from the stroke stack
		for (unsigned int i = 0; i < Strokes->size(); i++)
		{
			if((*Strokes)[i].ID == UndoList.back().ID)
			{
				Strokes->erase(Strokes->begin() + i);
				break;
			}
		}

		// save the info in the redo list
		RedoList.push_back(UndoList.back());

		//pop the element from the undo stack
		UndoList.pop_back();
	}
}

void Brush2D::redo()
{
	if(RedoList.size() > 0)
	{
		//get the last element and push it back on the stroke stack
		Strokes->push_back(RedoList.back());
		// also push it on the undo stack
		UndoList.push_back(RedoList.back());
		//pop it from the redo stack
		RedoList.pop_back();
	}
}

int Brush2D::getStrokeID()
{
	CurrentStrokeID = (CurrentStrokeID + 1) % 1000000000;
	return CurrentStrokeID;
}
