/*
This class provides free hand brush tool. Invoke tool with closed right hand and adjust brush size with closed left hand.

*/
#pragma once
#include "Tool.h"
#include "KinectPointer.h"
#include "Kinect Test\Stroke.h"
#include "Manipulator2D.h"
#include "Tools\Colorpicker.h"
#include "UndoRedoManager.h"


class Brush2D : public Tool
{
public:

	Brush2D(KinectPointer* right_hand, std::vector<Stroke>* strokes, Manipulator2D* manipulator);
	~Brush2D();

	void updateLeftHandInput(Pose &pose); 
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose); 

	void update(Pose &pose);
	void render();
	void undo();
	void redo();

	int getTextureID(){return TextureID;};
	// returns a new global stroke id
	static int getStrokeID();
	

private:
	// prepares a new stroke or adds a finished stroke into the stroke list
	void checkStroke();
	// sets brush size
	void setLineWidth(Pose& pose);
	// sets a new stroke point in the current stroke
	void setStrokePoint();

	// pointer to hand pointer
	KinectPointer* RightHandPointer;
	// pointer to stroke list
	std::vector<Stroke>* Strokes;
	// pointer to manipulator (to obtain local coordinates)
	Manipulator2D* Manipulator;

	float LineWidth;

	// maintain pose from last iteration step
	Pose LastPose;
	// maintain stroke point from last iteration step
	math::Vec2f LastPoint;
	
	// do we have drawed already
	bool Drawed;
	
	// undo/redo lists
	std::vector<Stroke> UndoList;
	std::vector<Stroke> RedoList;

	// current global stroke id
	static int CurrentStrokeID;
	// texture for tool icon
	int TextureID;
};