#include "Camera.h"



Camera::Camera(WindowProperties* window_props)
{
	ZNear = 0.1f;
	ZFar  = 1000.0f;
	FoV   = 45.0f;
	WindowProps = window_props;
}

Camera::~Camera()
{
}

void Camera::update()
{
	// calculate projection matrix
	float focal_length = 1.0f / (tanf(0.5f * FoV));
	float clipping_plane_dist = ZFar - ZNear;
	
	ProjectionMatrix.ma[ 0] = focal_length;
	ProjectionMatrix.ma[ 1] = 0;
	ProjectionMatrix.ma[ 2] = 0;
	ProjectionMatrix.ma[ 3] = 0;
	ProjectionMatrix.ma[ 4] = 0;
	ProjectionMatrix.ma[ 5] = focal_length * WindowProps->WindowAspect;
	ProjectionMatrix.ma[ 6] = 0;
	ProjectionMatrix.ma[ 7] = 0;
	ProjectionMatrix.ma[ 8] = 0;
	ProjectionMatrix.ma[ 9] = 0;
	ProjectionMatrix.ma[10] = -(ZNear + ZFar)/clipping_plane_dist;
	ProjectionMatrix.ma[11] = -1;
	ProjectionMatrix.ma[12] = 0;
	ProjectionMatrix.ma[13] = 0;
	ProjectionMatrix.ma[14] = -2*ZNear*ZFar/clipping_plane_dist;
	ProjectionMatrix.ma[15] = 0;


	// set the view matrix...
	ViewMatrix = TransformationMatrix;

	// ... and invert
	//viewMatrix.invert();
	
	math::Vec3f translation = ViewMatrix.getTranslation();
	math::Matrix44f rotation = ViewMatrix.getOrientation();
	rotation.transpose();
	ViewMatrix = math::Matrix44f::TranslationMatrix( -translation ) * rotation;
}


math::Matrix44f &Camera::getViewMatrix()
{
	return ViewMatrix;
}


math::Matrix44f &Camera::getProjectionMatrix()
{
	return ProjectionMatrix;
}