#include "OrbitManipulator.h"
#include <iostream>
#include "SOIL.h"

OrbitManipulator::OrbitManipulator()
{
	init(math::Vec3f(128.0f, 0.0f, 128.0f), 50.0f, 50.0f, 300.0f, 100.0f, 0);
}

OrbitManipulator::OrbitManipulator(Camera *camera)
{
	init(math::Vec3f(128.0f, 0.0f, 128.0f), 50.0f, 50.0f, 300.0f, 100.0f, camera);
}

OrbitManipulator::~OrbitManipulator()
{
}

void OrbitManipulator::init(math::Vec3f look_at, float azimuth, float elevation, float distance, float scaling, Camera *camera)
{
	LookAt = look_at;
	Azimuth = azimuth;
	Elevation = elevation;
	Distance = distance;
	Scaling = scaling;
	setCamera(camera);

	TextureID = SOIL_load_OGL_texture("../images/camera_3d.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
}

void OrbitManipulator::setCamera(Camera *camera)
{
	pCamera = camera;
	updateCamera();
}


void OrbitManipulator::updateLeftHandInput(Pose &pose)
{
	update(pose);
}

void OrbitManipulator::updateRightHandInput(Pose &pose)
{
	math::Vec3f pos = pose.RightHandPosition;
	rotate(Scaling*(pos.x - LastPose.RightHandPosition.x), Scaling*(LastPose.RightHandPosition.y - pos.y));
	
	update(pose);
}

void OrbitManipulator::updateBothHandInput(Pose &pose)
{

	zoom(Distance*(LastPose.DistanceHandToHand - pose.DistanceHandToHand));

	math::Vec3f centroid = 0.5 * (pose.LeftHandPosition + pose.RightHandPosition);
	math::Vec3f old_centroid = 0.5 * (LastPose.LeftHandPosition + LastPose.RightHandPosition);
	pan(Scaling*(centroid.x - old_centroid.x), Scaling*(centroid.y - old_centroid.y));

	//rotate(pose.AngleTransversalHandToHandVector - LastPose.AngleTransversalHandToHandVector, pose.AngleCoronalHandToHandVector - LastPose.AngleCoronalHandToHandVector);
	
	update(pose);
}

void OrbitManipulator::update(Pose &pose)
{
	updateCamera();
	LastPose = Pose(pose);
}

void OrbitManipulator::render()
{
}

void OrbitManipulator::updateCamera()
{
	// we compute the transformation matrix from look at point and polar coordinates
	math::Matrix44f m = math::Matrix44f::Identity();

	m.translate(LookAt);

	m.rotateY(math::degToRad(Azimuth));
	m.rotateX(math::degToRad(Elevation));
	m.translate(math::Vec3f( 0.0f, 0.0f, Distance));

	if( pCamera )
	{
		pCamera->TransformationMatrix = m;
		pCamera->update();
	}

	
}


void OrbitManipulator::pan(float delta_x, float delta_y)
{
	LookAt += (-delta_x * Distance * 0.01f) * pCamera->TransformationMatrix.getRight();
	LookAt += (-delta_y * Distance * 0.01f) * pCamera->TransformationMatrix.getUp();
}


void OrbitManipulator::rotate(float delta_azimuth, float delta_elevation)
{
	Azimuth   += delta_azimuth;
	Elevation += delta_elevation;
}


void OrbitManipulator::zoom(float delta_distance)
{
	Distance += delta_distance;
}


void OrbitManipulator::setLookAt(float x, float y, float z)
{
	LookAt = math::Vec3f(x, y, z);
}

