/*
This tool provides the generation of microstructures on a virtual landscape by randomly perturbating the heights of grid points.
The closed right hand invokes the tool and sets its position. The left hand adjusts size and strength of the tool.
*/
#pragma once

#include "Tool.h"
#include "math\Vec3f.h"
#include "KinectPointer.h"
#include "KinectTerrain\GridMesh.h"
#include "random_mod.h"

class RandomizeBrush: public Tool
{
public:
	RandomizeBrush(KinectPointer* right_hand, GridMesh* mesh);
	virtual ~RandomizeBrush(void);

	void updateLeftHandInput(Pose &pose);
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose);

	void update(Pose &pose);
	void render();
	void undo(){}
	void redo(){}

	// return texture id for editor icon
	int getTextureID(){return TextureID;}




private:
	// sets the tool parameters
	void setRadiusAndStrength(Pose& pose);
	// actual tool operation
	void processMesh();

	// pointer to hand pointer
	KinectPointer* RightHandPointer;
	// pointer to mesh
	GridMesh* Mesh;
	// maintain pose from last iteration step
	Pose LastPose;

	// brush parameters
	float BrushRadius;
	float BrushStrength;
	float Scale;
	// show brush if inside grid
	bool showBrush;
	// local coords of brush tip
	math::Vec3f BrushCoordinates;
	
	// tool icon texture
	int TextureID;

	// random numbers
	math::random RandomNumberGenerator;
};

