/*
Tool providing piecewise erasing of strokes. Invoke rubber with right hand, change its radius with left hand.

*/
#pragma once
#include "Tool.h"
#include "KinectPointer.h"
#include "Kinect Test\Stroke.h"
#include "Tools\Colorpicker.h"
#include "Manipulator2D.h"
#include "Brush2D.h"
#include "UndoRedoManager.h"

// helper struct for undo action
struct RubberInfo
{
	Stroke OldStroke;
	std::vector<Stroke> NewStrokes;
};


class Rubber2D : public Tool
{
public:

	Rubber2D(KinectPointer* right_hand, Manipulator2D* manipulator, std::vector<Stroke>* strokes, int* selected_stroke_id);
	~Rubber2D();

	void updateLeftHandInput(Pose &pose); 
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose); 

	void update(Pose &pose);
	void render();
	void undo();
	void redo();

	int getTextureID(){return TextureID;};

private:
	// erases a piece of the stroke in respect to rubber radius
	void rub();
	// sets the size of the rubber
	void setRubberSize(Pose& pose);
	// returns circle intersection point of a point outside the rubber and rubber center
	math::Vec2f getIntersectionPoint(math::Vec2f& point, math::Vec2f& rubber_center, float rubber_radius);
	// returns true if point lies inside the rubber circle
	bool insideRubber(math::Vec2f& point, math::Vec2f& rubber_center, float rubber_radius);
	// orthogonal projection of rubber center onto stroke segment
	math::Vec2f snapOntoStroke(math::Vec2f& point1, math::Vec2f& point2, math::Vec2f& rubber_center);

	// pointer to kinect pointer
	KinectPointer* RightHandPointer;
	// pointer to manipulator (to obtain local coordinates)
	Manipulator2D* Manipulator;
	// pointer to stroke list
	std::vector<Stroke>* Strokes;
	// pointer to selected stroke
	int* SelectedStrokeID;
	// size of rubber
	float RubberRadius;
	// maintain pose from last iteration step
	Pose LastPose;

	// undo/redo lists
	std::vector<RubberInfo> UndoList;
	std::vector<RubberInfo> RedoList;
	// tool icon texture
	int TextureID;
};