/*
Tool to draw straight lines. You can set the starting coordinates by pointing to a specific place 
on the canvas and closing the right hand. The place where you open the hand again will be the end 
coordinates of the line. The closed left hand controls the size of the line.

*/
#pragma once

#include "Tool.h"
#include "KinectPointer.h"
#include "Kinect Test\Stroke.h"
#include "Manipulator2D.h"
#include "Tools\Colorpicker.h"
#include "Tools\Brush2D.h"
#include "UndoRedoManager.h"

class StraightLine2D : public Tool
{
public:

	StraightLine2D(KinectPointer* right_hand, std::vector<Stroke>* strokes, Manipulator2D* manipulator);
	~StraightLine2D();

	void updateLeftHandInput(Pose &pose); 
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose); 

	void update(Pose &pose);
	void render();
	void undo();
	void redo();

	int getTextureID(){return TextureID;};

private:
	// prepares a new stroke or adds a finished stroke into the stroke list
	void checkStroke();
	// sets the line width
	void setLineWidth(Pose& pose);
	// sets the coordinates of the stroke end point
	void setStrokePoint();

	// pointer to kinect pointer
	KinectPointer* RightHandPointer;
	// pointer to stroke list
	std::vector<Stroke>* Strokes;
	// pointer to manipulator (to obtain local coordinates)
	Manipulator2D* Manipulator;

	// stroke line width
	float LineWidth;

	// maintain pose from last iteration step
	Pose LastPose;
	// start and end coordinates of current stroke
	math::Vec2f StartPoint;
	math::Vec2f EndPoint;

	// undo/redo lists
	std::vector<Stroke> UndoList;
	std::vector<Stroke> RedoList;

	// do we have drawed already
	bool Drawed;
	// tool icon texture
	int TextureID;
};